/*----------------------------------------------------------------------------
* Name:    mielib.c
* Purpose: MAPLE BUS sample device usage for STM32
* Version: V1.00
*----------------------------------------------------------------------------
* This file is part of the OnAMAPLE tools.
*
* Copyright (c) 2007-2021 r.a.m. Factory STORE. All rights reserved.
*----------------------------------------------------------------------------*/

#include <stm32f10x.h>                        // STM32F10x Library Definitions
#include "STM32_Init.h"                           // STM32 Initialization

#include "misc.h"
#include <string.h>
#include "../MIELIB/inc/maplebus.h"
#include "../MIELIB/inc/interfaces.h"
#include "../MIELIB/inc/board.h"
#include "./PS_2.h"
#include "./mouse.h"

#define	BUT_MIDDLE_BIT	(1 << 2)
#define	BUT_RIGHT_BIT		(1 << 1)
#define	BUT_LEFT_BIT		(1 << 0)

#define	X_SIGN_BIT		(1 << 4)
#define	Y_SIGN_BIT		(1 << 5)

#define	X_OVF_BIT			(1 << 6)
#define	Y_OVF_BIT			(1 << 7)

unsigned char PS_2_MPacket[4];

/*----------------------------------------------------------------------------
READING Mouse Data and convert it into Maple Controller data
*----------------------------------------------------------------------------*/
void assigndata_cont(LPCONTROLLERCONDITION ContCond)
{
//							X			Ra,La,Da,Ua,St,A,B	
//0b00000000	0b00000000	0b00000100	0b11111110	
	if(PS_2_MPacket[1] > 50)
	{
		if(PS_2_MPacket[0] & X_SIGN_BIT)
			ContCond->ButtonsLO &= ~CLa_BUTTON;
		else
			ContCond->ButtonsLO &= ~CRa_BUTTON;
	}

	if(PS_2_MPacket[2] > 50)
	{
		if(PS_2_MPacket[0] & Y_SIGN_BIT)
			ContCond->ButtonsLO &= ~CDa_BUTTON;
		else
			ContCond->ButtonsLO &= ~CUa_BUTTON;
	}
	
	/* Additional onboard button as START Button */
	ContCond->ButtonsLO &= (!(GPIOA->IDR & (1<<15))) ? ~CST_BUTTON : 0xFFFF;
	
	ContCond->ButtonsLO &= (PS_2_MPacket[0] & BUT_LEFT_BIT) ? ~CA_BUTTON : 0xFFFF;
	ContCond->ButtonsLO &= (PS_2_MPacket[0] & BUT_RIGHT_BIT) ? ~CB_BUTTON : 0xFFFF;
	ContCond->ButtonsLO &= (PS_2_MPacket[0] & BUT_MIDDLE_BIT) ? ~CST_BUTTON : 0xFFFF;

	ContCond->ButtonsHI &= (PS_2_MPacket[0] & BUT_LEFT_BIT) ? ~CA_BUTTON : 0xFFFF;
	ContCond->ButtonsHI &= (PS_2_MPacket[0] & BUT_RIGHT_BIT) ? ~CB_BUTTON : 0xFFFF;
	ContCond->ButtonsHI &= (PS_2_MPacket[0] & BUT_MIDDLE_BIT) ? ~CST_BUTTON : 0xFFFF;
	
	return;
}

/*----------------------------------------------------------------------------
READING Mouse Data and convert it into Maple Mouse data
*----------------------------------------------------------------------------*/
void assigndata_mouse(LPMOUSECONDITION MouseCond)
{
//		Generic PS/2 Mouse Packet Bits
//		BYTE	7		6		5		4		3		2		1		0
//		0			yo	xo	ys	xs	ao	bm	br	bl
//		1									xm
//		2									ym
//		Code	Description
//		yo	Y-Axis Overflow
//		xo	X-Axis Overflow
//		ys	Y-Axis Sign Bit (9-Bit Y-Axis Relative Offset)
//		xs	X-Axis Sign Bit (9-Bit X-Axis Relative Offset)
//		ao	Always One
//		bm	Button Middle (Normally Off = 0)
//		br	Button Right (Normally Off = 0)
//		bl	Button Left (Normally Off = 0)
//		xm	X-Axis Movement Value
//		ym	Y-Axis Movement Value
		
		/* Translate overflow bits */
		MouseCond->AOV |= (PS_2_MPacket[0] & X_OVF_BIT) ? AOV0 : 0;
		MouseCond->AOV |= (PS_2_MPacket[0] & Y_OVF_BIT) ? AOV1 : 0;
		/* AOV2 is never happens because scroll */
		MouseCond->AOV |= 0;
		
		/* Additional onboard button as START Button */
		MouseCond->BTN &= (!(GPIOA->IDR & (1<<15))) ? ~CST_BUTTON : 0xFFFF;

		/* Translate Buttons */
		MouseCond->BTN &= (PS_2_MPacket[0] & BUT_LEFT_BIT) ? ~MA_BUTTON : 0xFFFF;
		MouseCond->BTN &= (PS_2_MPacket[0] & BUT_RIGHT_BIT) ? ~MB_BUTTON : 0xFFFF;
		MouseCond->BTN &= (PS_2_MPacket[0] & BUT_MIDDLE_BIT) ? ~MW_BUTTON : 0xFFFF;
		
		/* Translate X/Y and Wheel */
		MouseCond->AC1 = (PS_2_MPacket[1] << 1);	//X-axis analog coordinate 8bit to 9bit convert
		MouseCond->AC1 |= (PS_2_MPacket[0] & X_SIGN_BIT) ? (1 << 9) : 0;// Add sign for X Axis
		
		MouseCond->AC2 = (PS_2_MPacket[2] << 1);	//Y-axis analog coordinate 8bit to 9bit convert
		MouseCond->AC2 |= (PS_2_MPacket[0] & Y_SIGN_BIT) ? (1 << 9) : 0;// Add sign for Y Axis
		
		MouseCond->AC3 = (PS_2_MPacket[3] << 1);	//Z-axis analog coordinate (wheel) convert 8bit to 10bit
		
		return;
}

bool bReDetect;

void mouse_callback(void)
{
	static uint8_t bCounter = 0;
	
	if(!bReDetect)
	{
		PS2_Status = PS_2_CMDMODE; 
		PS2_Send_Cmd(READ_DATA);
		
		if(PS2_Get_Byte() == 0xFA)
		{
			PS_2_MPacket[0] = PS2_Get_Byte();
			PS_2_MPacket[1] = PS2_Get_Byte();
			PS_2_MPacket[2] = PS2_Get_Byte();
			PS_2_MPacket[3] = PS2_Get_Byte();
			
			bCounter = 0;
		}
		else
		{
			bCounter ++;
			
			if(bCounter == 20)
				bReDetect = true;
		}
	}
}

/*----------------------------------------------------------------------------
MAIN function
*----------------------------------------------------------------------------*/
int main(void) {

	unsigned char bNumbersOfTramsmits = 3;
	int bRet = 0;
	stm32_Init();                      // STM32 setup
	delay_init(72 / 8);

	/* Initialize OnMAPLES-EB */
	InitBOARD(INIT_RCC | INIT_GPIO | INIT_XMCLK | INIT_XMEXT);

	/* Enable All LEDs just for test */
	LED_MTRM_ON();
	LED_MRCV_ON();
	LED_RD_ON();
	LED_WR_ON();

	if (GetXMVer() == 0x00)
	{
		/*No XMAPLE CPLD Detected*/
		while (1); /*HALT*/
	}

redetect:
	bReDetect = true;
	/* Disable MIE Small Logic Companion */
	DisMIECOMP();
	/* Disable Interrupts processing */
	DisableMapleINTs();

	/* Disable All LEDs */
	LED_MTRM_OFF();
	LED_MRCV_OFF();
	LED_RD_OFF();
	/* Signalling "No mouse" */
	LED_WR_ON();

	/* Clear mouse report */
	memset(PS_2_MPacket, 0x00, 6);
	
	EnableMaple_nEOP();

	/* Wait for mouse connect */
	while (1)
	{
		bRet = Init_Mouse();

		if (!bRet)
			break;
	}

	/* Signalling "Mouse Attached" */
	LED_WR_OFF();
	LED_RD_ON();
	bReDetect = false;

	while (1)
	{
		/* No Mouse Connected Detect */
		if(bReDetect)
			goto redetect;
		
		if (MapleReceive(&inMapleFrame) == RCV_OK)
		{
			LED_MTRM_ON();
			bNumbersOfTramsmits = 3;
			ProcessDeviceFrame(&inMapleFrame);
			LED_MTRM_OFF();
		}
		else
		{
			if (bNumbersOfTramsmits != 0)
			{
				bNumbersOfTramsmits--;
				inMapleFrame.Command = TransmitAgain;
				inMapleFrame.DestAP = inMapleFrame.OriginAP;
				if (m_bOurDeviceAddress != 0xFF)
					inMapleFrame.OriginAP = m_bOurDeviceAddress;
				else
				{
					//We do not receive address for this device
				}
				inMapleFrame.DataSize = 0;
				MapleSend(&inMapleFrame, NO_TIMEOUT_CHECK);
			}
		}
	}

} // end main
